#include "msg.hpp"

#define DECLARE_TEMPLATE_OBJECT(_obj, _type)	\
	json::Object _obj;							\
	makeTemplate(_obj, _type);					\

namespace Message
{
	// parser
	void Parser::parse(const std::string & message)
	{
		mObject = json::Deserialize(message);
	}

	std::string Parser::type() const
	{
		return get<std::string>("type");
	}

	// request
	void Request::setUserID(const std::string & id)
	{
		mID = id;
	}

	const std::string & Request::getUserID()
	{
		return mID;
	}

	void Request::makeTemplate(json::Object & obj, const std::string & type) const
	{
		obj["id"] = mID;
		obj["type"] = type;
	}

	std::string Request::command(const std::string & cmd) const
	{
		DECLARE_TEMPLATE_OBJECT(obj, "command");

		obj["command"] = cmd;

		return json::Serialize(obj);
	}

	std::string Request::login(const std::string & pw) const
	{
		DECLARE_TEMPLATE_OBJECT(obj, "login");

		obj["pw"] = mID + "$" + pw;

		return json::Serialize(obj);
	}

	std::string Request::chat(const std::string & to, const std::string & sessid, const std::string & message) const
	{
		DECLARE_TEMPLATE_OBJECT(obj, "chat");

		obj["from"   ] = mID;
		obj["to"     ] = to;
		obj["sess-id"] = sessid;
		obj["message"] = message;

		return json::Serialize(obj);
	}

	std::string Request::friendList() const
	{
		DECLARE_TEMPLATE_OBJECT(obj, "friend-list");

		return json::Serialize(obj);
	}

	// response
	void Response::makeTemplate(json::Object & obj, const std::string & type) const
	{
		obj["type"] = type;
	}

	std::string Response::result(const bool status) const
	{
		DECLARE_TEMPLATE_OBJECT(obj, "result");

		obj["result"] = status;

		return json::Serialize(obj);
	}
};
