
#include "msgclnt.h"
#include "msg.hpp"

#include <stdlib.h>

// --------------------------------------------------------------

typedef struct SWorkerArg
{
	cf::network::tcp * socket;
	SCallback * callback;
} SWorkerArg;

int worker(void * arg)
{
	SWorkerArg * inst = (SWorkerArg *) arg;

	bool isError = false;
	Message::Parser parser;

	while (true)
	{
		try
		{
			std::string message = inst->socket->receive().toString();
			parser.parse(message);

			isError = false;
		}
		catch (cf::exception & e)
		{
			LOG(e.stackTrace());

			isError = true;
		}

		if (parser.type() == "result")
		{
			SResultResponse recv;

			recv.result  = parser.get<bool>("result");
			recv.isError = isError;

			inst->callback->result(recv);
		}
		else if (parser.type() == "chat")
		{
			SReceivedMessage recv;

			recv.sessid  = parser.get<std::string>("sess-id");
			recv.sender  = parser.get<std::string>("from");
			recv.message = parser.get<std::string>("message");
			recv.isError = isError;

			inst->callback->message(recv);
		}
	}

	free(arg);

	return 0;
}

// --------------------------------------------------------------

chev::chev()
	: listener(worker)
{
}

bool chev::connect(const std::string & host, const unsigned short port)
{
	try
	{
		socket.connect(host, port);

		return true;
	}
	catch (cf::exception & e)
	{
		LOG(e.what());

		return false;
	}
}

bool chev::login(const std::string & id, const std::string & pw)
{
	try
	{
		request.setUserID(id);

		socket.send(request.login(pw));

		std::string response = socket.receive().toString();
		Message::Parser parser;
		parser.parse(response);

		return parser.get<bool>("result");
	}
	catch (cf::exception & e)
	{
		LOG(e.what());

		return false;
	}
}

bool chev::runReceiver(const SCallback & cb)
{
	callback = cb;

	SWorkerArg * arg = (SWorkerArg *)malloc(sizeof(SWorkerArg));
	if (!arg)
		return false;

	arg->socket = &socket;
	arg->callback = &callback;

	listener.start(arg);

	return true;
}

bool chev::sendMessage(const std::string & friendid, const std::string & sessid, const std::string & message)
{
	try
	{
		socket.send(request.chat(friendid, sessid, message));

		std::string response = socket.receive().toString();
		Message::Parser parser;
		parser.parse(response);

		return parser.get<bool>("result");
	}
	catch (cf::exception & e)
	{
		LOG(e.what());

		return false;
	}
}
